==> support-compat-25.0.0-sources.jar: android/support/v4/graphics/drawable/DrawableCompatJellybeanMr1.java <==
/*
 * Copyright (C) 2015 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package android.support.v4.graphics.drawable;

import android.graphics.drawable.Drawable;
import android.util.Log;
import android.widget.CompoundButton;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * Implementation of drawable compatibility that can call Jellybean MR1 APIs.
 */
class DrawableCompatJellybeanMr1 {

    private static final String TAG = "DrawableCompatJellybeanMr1";

    private static Method sSetLayoutDirectionMethod;
    private static boolean sSetLayoutDirectionMethodFetched;

    private static Method sGetLayoutDirectionMethod;
    private static boolean sGetLayoutDirectionMethodFetched;

    public static boolean setLayoutDirection(Drawable drawable, int layoutDirection) {
        if (!sSetLayoutDirectionMethodFetched) {
            try {
                sSetLayoutDirectionMethod =
                        Drawable.class.getDeclaredMethod("setLayoutDirection", int.class);
                sSetLayoutDirectionMethod.setAccessible(true);
            } catch (NoSuchMethodException e) {
                Log.i(TAG, "Failed to retrieve setLayoutDirection(int) method", e);
            }
            sSetLayoutDirectionMethodFetched = true;
        }

        if (sSetLayoutDirectionMethod != null) {
            try {
                sSetLayoutDirectionMethod.invoke(drawable, layoutDirection);
                return true;
            } catch (Exception e) {
                Log.i(TAG, "Failed to invoke setLayoutDirection(int) via reflection", e);
                sSetLayoutDirectionMethod = null;
            }
        }
        return false;
    }

    public static int getLayoutDirection(Drawable drawable) {
        if (!sGetLayoutDirectionMethodFetched) {
            try {
                sGetLayoutDirectionMethod = Drawable.class.getDeclaredMethod("getLayoutDirection");
                sGetLayoutDirectionMethod.setAccessible(true);
            } catch (NoSuchMethodException e) {
                Log.i(TAG, "Failed to retrieve getLayoutDirection() method", e);
            }
            sGetLayoutDirectionMethodFetched = true;
        }

        if (sGetLayoutDirectionMethod != null) {
            try {
                return (int) sGetLayoutDirectionMethod.invoke(drawable);
            } catch (Exception e) {
                Log.i(TAG, "Failed to invoke getLayoutDirection() via reflection", e);
                sGetLayoutDirectionMethod = null;
            }
        }
        return -1;
    }

}

==> support-compat-25.2.0-sources.jar: android/support/v4/graphics/drawable/DrawableCompatJellybeanMr1.java <==
/*
 * Copyright (C) 2015 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package android.support.v4.graphics.drawable;

import android.graphics.drawable.Drawable;
import android.support.annotation.RequiresApi;
import android.annotation.TargetApi;
import android.util.Log;

import java.lang.reflect.Method;

/**
 * Implementation of drawable compatibility that can call Jellybean MR1 APIs.
 */

@RequiresApi(17)
@TargetApi(17)
class DrawableCompatJellybeanMr1 {

    private static final String TAG = "DrawableCompatJellybeanMr1";

    private static Method sSetLayoutDirectionMethod;
    private static boolean sSetLayoutDirectionMethodFetched;

    private static Method sGetLayoutDirectionMethod;
    private static boolean sGetLayoutDirectionMethodFetched;

    public static boolean setLayoutDirection(Drawable drawable, int layoutDirection) {
        if (!sSetLayoutDirectionMethodFetched) {
            try {
                sSetLayoutDirectionMethod =
                        Drawable.class.getDeclaredMethod("setLayoutDirection", int.class);
                sSetLayoutDirectionMethod.setAccessible(true);
            } catch (NoSuchMethodException e) {
                Log.i(TAG, "Failed to retrieve setLayoutDirection(int) method", e);
            }
            sSetLayoutDirectionMethodFetched = true;
        }

        if (sSetLayoutDirectionMethod != null) {
            try {
                sSetLayoutDirectionMethod.invoke(drawable, layoutDirection);
                return true;
            } catch (Exception e) {
                Log.i(TAG, "Failed to invoke setLayoutDirection(int) via reflection", e);
                sSetLayoutDirectionMethod = null;
            }
        }
        return false;
    }

    public static int getLayoutDirection(Drawable drawable) {
        if (!sGetLayoutDirectionMethodFetched) {
            try {
                sGetLayoutDirectionMethod = Drawable.class.getDeclaredMethod("getLayoutDirection");
                sGetLayoutDirectionMethod.setAccessible(true);
            } catch (NoSuchMethodException e) {
                Log.i(TAG, "Failed to retrieve getLayoutDirection() method", e);
            }
            sGetLayoutDirectionMethodFetched = true;
        }

        if (sGetLayoutDirectionMethod != null) {
            try {
                return (int) sGetLayoutDirectionMethod.invoke(drawable);
            } catch (Exception e) {
                Log.i(TAG, "Failed to invoke getLayoutDirection() via reflection", e);
                sGetLayoutDirectionMethod = null;
            }
        }
        return -1;
    }

}
